@@ -10,8 +10,8 @@
 #' @param folders character. Folder(s) where the data is located (each folder represents a scenario)
 #' @param names character. Scenario names
 #'
-#' @seealso \code{\link{plexos_close}}
-#' @seealso \code{\link{query_master}}
+#' @seealso \code{\link{query_master}} to perform standard queries of data
+#' @seealso \code{\link{query_sql}} to perform custom queries
 #' 
 #' @export
 plexos_open <- function(folders = ".", names = folders) {
@@ -20,7 +20,7 @@ plexos_open <- function(folders = ".", names = folders) {
   
   # Check for wildcard
   if (length(folders) == 1L) {
-    if (folders == "*") {
+    if (identical(folders, "*")) {
       folders <- list_folders()
       names <- folders
     }
@@ -31,7 +31,7 @@ plexos_open <- function(folders = ".", names = folders) {
   
   # Change default scenario name to something better than '.'
   if (length(folders) == 1L) {
-    if ((folders == ".") & (names == ".")) {
+    if (identical(folders, ".") & identical(names, ".")) {
       names <- "default"
     }
   }
@@ -52,7 +52,8 @@ plexos_open <- function(folders = ".", names = folders) {
   
   # Get database file names
   df <- data.frame(folder = folders,
-                   scenario = factor(names, levels = names)) %>%
+                   scenario = factor(names, levels = names),
+                   stringsAsFactors = FALSE) %>%
         rowwise() %>%
         do(plexos_list_files(.))
   
@@ -76,7 +77,9 @@ plexos_open <- function(folders = ".", names = folders) {
     ungroup() %>%
     mutate(position = 1:n()) %>%
     group_by(scenario, position, filename) %>%
-    do(db = src_sqlite(as.character(.$filename)))
+    do(tables = get_list_tables(.$filename),
+       properties = get_table(.$filename, "property")) %>%
+    ungroup()
   
   # Add rplexos class to object
   class(out) <- c("rplexos", class(out))
@@ -107,72 +110,13 @@ plexos_open <- function(folders = ".", names = folders) {
   out
 }
 
-#' Close all PLEXOS databases
-#'
-#' Close all the open connections to PLEXOS SQLite databases. This function
-#' completely erases the provided object.
-#'
-#' @param db PLEXOS database object
-#'
-#' @seealso \code{\link{plexos_open}} to create the PLEXOS database object
-#'
-#' @export
-plexos_close <- function(db) {
-  assert_that(is.rplexos(db))
-
-  # For each database, close the connection
-  db %>%
-    do(result = dbDisconnect(.$db$con))
-  
-  # Remove object from memory
-  rm(list = deparse(substitute(db)), envir = sys.frame(-1))
-  
-  TRUE
-}
-
-# Create custom summary for rplexos objects
-#' @export
-#' @method summary rplexos
-summary.rplexos <- function(object, ...) {
-  info <- object %>%
-    group_by(position, scenario, filename) %>%
-    summarise(tables = length(src_tbls(db[[1]]))) %>%
-    as.data.frame
-  
-  # Query config to get rplexos and PLEXOS version
-  conf <- query_config(object) %>%
-    select(position, PLEXOS = Version, rplexos)
-  
-  # Join the two tables
-  info2 <- info %>% inner_join(conf, by = "position")
-  
-  # Print table
-  print(info2, row.names = FALSE)
-}
-
 # Create custom visualization for rplexos objects
 #' @export
 print.rplexos <- function(x, ...) {
-  cat("Structure:\n")
-  summary(x)
-  
-  cat("\nTables:\n")
-  info <- x %>%
-    group_by(position) %>%
-    do(data.frame(table = src_tbls(.$db[[1]])))
-  
-  print(reshape2::dcast(info, table ~ position, fun.aggregate = length, value.var = "table"),
-        row.names = FALSE)
-}
-
-# Custom ungroup method, to preserve 'rplexos' class
-ungroup.rplexos <- function(x) {
-  class(x) <- setdiff(class(x), c("grouped_df", "rowwise_df"))
-  x
-}
-
-# Avoid group_by_.rowwise_df warning
-group_by_.rplexos <- function(.data, ...) {
-  class(.data) <- setdiff(class(.data), c("rplexos", "rowwise_df"))
-  group_by_(.data, ...)
+  out <- x %>%
+    group_by(scenario, position, filename) %>%
+    summarize(tables = nrow(tables[[1]]),
+              properties = nrow(properties[[1]])) %>%
+    ungroup() %>%
+    print()
 }
